from . import utils
from datetime import datetime
import os, sys, jinja2

DEFAULT_CONFIG = """name: A PyBlog Blog
tagline: Demoing quick-and-dirty python blog generation
root_url: https://blog.com
"""

class Page:
    title           = u''
    content         = u''
    template        = None
    slug            = u''
    url             = u''
    date            = datetime.now()
    
    def __init__(self, dir, file_path):
        """Creates a generic page from a file"""
        parts = utils.file_get(dir, file_path).split('\n\n', 1)
        if len(parts) == 2:
            meta = utils.parse_headers(parts[0])
            self.content = parts[1]
            self.title = meta['title']
            if 'template' in meta:
                self.template = meta['template'] 
            if 'date' in meta:
                self.date = datetime.strptime(meta['date'], '%Y-%m-%d %H:%M:%S')
            else:
                self.date = utils.file_mtime(dir, file_path)
        else:
            self.content = parts[0]
            self.date = utils.file_mtime(dir, file_path)
    
    @staticmethod
    def make_post(blog, file):
        """Creates a post object"""
        post = Page(blog.posts_dir, file)
        post.slug = utils.slugify(post.title)
        post.url = post.date.strftime('%Y/')+post.slug+'.html'
        return post
    
    @staticmethod
    def make_page(blog, file):
        """Creates a page object"""
        page = Page(blog.pages_dir, file)
        page.slug = file
        page.url = file
        tpl = blog.env.from_string(page.content)
        page.content = tpl.render(blog=blog)
        return page
    
    def __repr__(self):
        return 'Blog.Page(%s)' % self.slug

class Blog:
    name            = u''
    root_url        = u''
    tagline         = u''
    in_dir          = u''
    pages_dur       = u''
    posts_dir       = u''
    templates_dir   = u''
    static_dir      = u''
    out_dir         = u''
    env             = None
    
    def __init__(self, in_dir, out_dir):
        """Creates the blog object from the config file"""
        data = utils.file_get(in_dir, 'config.txt')
        options = utils.parse_headers(data)
        
        self.name           = options['name']
        self.tagline        = options['tagline']
        self.root_url       = options['root_url']
        
        self.templates_dir  = os.path.join(in_dir, '_templates')
        self.static_dir     = os.path.join(in_dir, '_static')
        self.pages_dir      = os.path.join(in_dir, '_pages')
        self.posts_dir      = os.path.join(in_dir, '_posts')
        self.in_dir         = in_dir
        self.out_dir        = out_dir
        
        utils.dir_make(out_dir)
        utils.dir_copy(self.static_dir, out_dir)
        self.env = utils.create_jinja_env(self.templates_dir)
    
    def get_posts(self):
        """returns a list of all posts objects"""
        files = utils.file_list(self.posts_dir, '.txt')
        return sorted([Page.make_post(self, file) for file in files],
            key=lambda x: x.date, reverse=True)
    
    def get_pages(self):
        """returns a list of all pages objects"""
        files = utils.file_list(self.pages_dir)
        return sorted([Page.make_page(self, file) for file in files],
            key=lambda x: x.date, reverse=True)
    
    def write_html(self, post):
        if(post.template):
            template = self.env.get_template(post.template)
            utils.file_put(self.out_dir, post.url, template.render(blog=self, post=post))
        else:
            utils.file_put(self.out_dir, post.url, post.content)
        
    
    @staticmethod
    def dir_init(dir):
        utils.dir_make(os.path.join(dir, '_posts'))
        utils.dir_make(os.path.join(dir, '_pages'))
        utils.dir_make(os.path.join(dir, '_static'))
        utils.dir_make(os.path.join(dir, '_templates'))
        utils.file_put(dir, 'config.txt', DEFAULT_CONFIG)
        pass
    
    @staticmethod
    def compile(src, dist):
        """Used for command-line usage, compiles the blog"""
        engine = Blog(src, dist)
        for post in engine.get_posts():
            engine.write_html(post)
        for page in engine.get_pages():
            engine.write_html(page)
    
    def __repr__(self):
        return 'Blog.Engine(%s)' % root_url
        


import markdown
import jinja2
import os
import re
from distutils import dir_util
from datetime import datetime

md = markdown.Markdown(extensions=[
    'markdown.extensions.fenced_code',
    'markdown.extensions.tables',
    'markdown.extensions.smarty',
	'markdown.extensions.codehilite(css_class=highlight, linenums=False)'
])

# Filesystem shortcuts

def dir_copy(src, dist):
    dir_util.copy_tree(src, dist)

def dir_make(path):
    if not os.path.exists(path):
        os.makedirs(path)

def file_get(dir, file_name):
    """Returns the UTF-8 decoded contents of a file"""
    with open(os.path.join(dir, file_name)) as f:
        return f.read().decode('utf-8')

def file_put(dir, file_name, contents):
    """Saves a utf-8 string to a file"""
    path = os.path.join(dir, file_name)
    dir_make(os.path.dirname(path))
    with open(path, 'w') as f:
        f.write(contents.encode('utf-8'))

def file_list(dir, ext=''):
    return [f for f in os.listdir(dir) if 
        os.path.isfile(os.path.join(dir, f)) and f.endswith(ext)]

def file_mtime(dir, file_name):
    datetime.fromtimestamp(os.path.getmtime(os.path.join(dir, file_name)))
        
# Blog utilities

def parse_headers(str):
    """Parses a HTTP-style headers"""
    headers = {}
    for line in str.split('\n'):
        key, value = line.split(': ', 1)
        headers[key] = value
    return headers

def create_jinja_env(tpl_path):
    env = jinja2.Environment(loader=jinja2.FileSystemLoader(tpl_path))
    env.filters['markdown'] = markdown
    env.filters['shortdate'] = shortdate
    env.filters['longdate'] = longdate
    env.filters['rssdate'] = rssdate
    env.filters['slugify'] = slugify
    return env

# Filters, used for jinja

def slugify(text):
    """Creates a url-valid slug fro a string"""
    slug = text.encode('ascii', 'ignore').lower()
    slug = re.sub(r'[^a-z0-9]+', '-', slug).strip('-')
    slug = re.sub(r'[-]+', '-', slug)
    return slug

def markdown(text):
    """Markdownify a piece of text"""
    return jinja2.Markup(md.convert(text))

def shortdate(t):
    """Returns a british-formatted date without the year"""
    return t.strftime('%b %d')

def longdate(t):
    """Returns a british-formatted date with the year"""
    return t.strftime('%b %d, %Y')

def rssdate(t):
    """Returns a RSS-valid date without the year"""
    return t.strftime('%a, %d %b %Y %H:%M:%S %z')


import pyblog
import sys

def pyblog_init():
    if len(sys.argv) != 2:
        sys.exit('usage: %s <directory>' % sys.argv[0])
    try:
        print 'creating blog structure...'
        pyblog.Blog.dir_init(sys.argv[1])
        print 'done'
    except Exception as e:
        sys.exit('error: %r' % e)

def pyblog_build():
    if len(sys.argv) != 3:
        sys.exit('usage: %s <input_dir> <output_dir>' % sys.argv[0])
    try:
        print 'compiling blog'
        pyblog.Blog.compile(sys.argv[1], sys.argv[2])
        print 'done'
    except Exception as e:
        sys.exit('error: %r' % e)



from setuptools import setup

setup(name='pyblog',
      version='0.3',
      description='Dirt-simple static blog general',
      url='http://github.com/cesarparent/pyblog',
      author='Cesar Parent',
      author_email='cesar@cesarparent.com',
      license='MIT',
      packages=['pyblog'],
      install_requires=[
          'markdown',
          'jinja2',
		  'pygments'
      ],
      entry_points = {
          'console_scripts': [
              'pyblog-init = pyblog.cli:pyblog_init',
              'pyblog-build = pyblog.cli:pyblog_build'
          ],
      },
      zip_safe=False)